import sys,os
from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
from tkinter import simpledialog
from tkinter import messagebox
from tkinter import filedialog
import yaml
from collections import OrderedDict
from amarettopy import *
import logging
from amarettopy.gui.util  import *

logger = logging.getLogger("amarettopy")

class InitTab(Tk.Frame):

    def __init__(self, amaretto, master=None):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        self.initButton= Tk.Button(self, text='Init', width= "10", command=self.init)
        self.initButton.pack(pady=5)

        self.calibButton = Tk.Button(self, text='Calib', width= "10", command=self.calib)
        self.calibButton .pack(pady=5)

        self.zeroButton= Tk.Button(self, text='SetZero', width= "10", command=self.setZero)
        self.zeroButton.pack(pady=5)

        self.resetButton= Tk.Button(self, text='Reset', width= "10", command=self.reset)
        self.resetButton.pack(pady=5)

        self.setFaultButton= Tk.Button(self, text='Fault', width= "10", command=self.fault_open)
        self.setFaultButton.pack(pady=5)

    def fault_open(self):
        
        logger.error("Fault button clicked")
        inputDialog = SelectFaultDialog(self)
        self.wait_window(inputDialog.top)

        if inputDialog.faults is None:
            return
        
        tryAmaretto(lambda: self.amaretto.fault(amarettoGuiInfo.targetDevId, inputDialog.faults))
        logger.error("Fault Error1 %s" % str(inputDialog.faults))

    def reset(self):
        logger.info("Reset button clicked ")
        tryAmaretto(lambda: self.amaretto.reset(amarettoGuiInfo.targetDevId))

    def calib(self):
        logger.info("Calib button clicked ")
        tryAmaretto(lambda: calibrate(self.amaretto, amarettoGuiInfo.targetDevId))

    def init(self):
        logger.info("Init button clicked ")

        def f():
            _f = os.path.join(os.path.dirname(__file__), "config/default_params.yml")
            self.amaretto.load_servo_params(amarettoGuiInfo.targetDevId, _f)
            self.amaretto.set_device_id(amarettoGuiInfo.targetDevId, 1)

        tryAmaretto(f)

    def setZero(self):
        logger.info("Zero button clicked ")

        def f():
            p = self.amaretto.query_servo_status(amarettoGuiInfo.targetDevId)[POS]
            self.set_sys_offset(amarettoGuiInfo.targetDevId, p % 65536)

        tryAmaretto(f)


class SelectFaultDialog:
    
    def __init__(self, parent):
        top = self.top = Tk.Toplevel(parent)
        top.grab_set()
        top.focus_set()

        self.faults = None

        self.vars = []
        self.checkButtons = []

        for (faultNo, faultName) in faultDict.items():
            var = Tk.BooleanVar()
            checkButton = Tk.Checkbutton(top, text=faultName, variable=var)
            checkButton.pack(anchor = "w")
            self.vars.append(var)
            self.checkButtons.append(checkButton)

        self.submitButton = Tk.Button(top, text='Submit', command=self.send)
        self.submitButton.pack()

    def send(self):
        self.faults = 0

        for i, var in enumerate(self.vars):
            if var.get() == True:
                self.faults += list(faultDict.keys())[i]

        self.top.destroy()




import sys,os
from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
from tkinter import simpledialog
from tkinter import messagebox
from tkinter import filedialog
import yaml
from collections import OrderedDict
from amarettopy import *
import logging
from amarettopy.gui.util  import *

logger = logging.getLogger("amarettopy")

class ConfigTab(Tk.Frame):

    def __init__(self, amaretto, configfile = 'params.yml', editable = False, master=None):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        yaml.add_constructor(yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
                             lambda loader, node: OrderedDict(loader.construct_pairs(node)))

        f = open(os.path.join(os.path.dirname(__file__), 'config', configfile))
        self.commands = yaml.load(f)
        self.editable = editable
        self.show_params()

    def modal_open(self, command):

        def x():

            answer = simpledialog.askstring("Input", command['message'],
                                            parent=self)
            if answer is None:
                return
            if answer.replace('.', '', 1).lstrip("-+").isnumeric() and self.amaretto.is_open():
                try:
                    method = "self.amaretto.set_" + command['method']+"(" + str(amarettoGuiInfo.targetDevId) + ", " + answer +")"
                    eval(method)
                    logger.info('parameter value edit to: %s'%answer)
                except MCPError as e:
                    messagebox.showerror('communication exception', str(e))
                    logger.error('communication exception: %s'% str(e))
                except Exception as e:
                    messagebox.showerror('exception',str(e))
                    logger.error('exception: %s'% str(e))
                self.show_params()
            else: 
                print('do nothing')
                logger.info('do nothing')
        return x

    def refresh(self):
        logger.info("Refresh button clicked")
        self.show_params()

    def show_params(self):
        list = self.slaves()
        for l in list:
            l.destroy()

        self.bottomFrame = Frame(self)
        self.bottomFrame.pack() 

        if self.editable:
            self.saveFileButton= Tk.Button(self.bottomFrame, text='SaveFile', command=self.save_servo_params)
            
            self.saveFileButton.grid(row=0, column=0, columnspan=1, padx=5, pady=5)
            
            self.loadFileButton= Tk.Button(self.bottomFrame, text='LoadFile', command=self.load_servo_params)
            
            self.loadFileButton.grid(row=0, column=1, columnspan=1, padx=5, pady=5)

            self.loadDefaultButton= Tk.Button(self.bottomFrame, text='LoadDefault', command=self.load_defaut_params)
            
            self.loadDefaultButton.grid(row=0, column=2, columnspan=1, padx=5, pady=5)

        self.button= Tk.Button(self.bottomFrame, text='Refresh', command=self.refresh)
        
        self.button.grid(row=0, column=3, columnspan=1, padx=5, pady=5)

        commands = self.commands
        errorOccured = False
        
        for a in commands:
            frame = Frame(self)
            frame.pack()
            param = '?'

            if self.amaretto.is_open() and not errorOccured:
                try:
                    method = "self.amaretto.get_" + commands[a]['method' ]+"(" + str(amarettoGuiInfo.targetDevId) + ")"
                    print(method)
                    logger.info(method)
                    param = str(eval(method)) + commands[a]['unit']
                    print(param)
                    logger.info(param)
                
                except Exception as e:
                    errorOccured = True

            label =Label(frame, text= commands[a]['name'], font=("Arial Bold", 10), width=30)
            label.pack(side=LEFT)
            label_param =Label(frame, text= param, width=30, font=("Arial Bold", 10))
            label_param.pack(side=LEFT)

            if self.editable:
                modal= Tk.Button(frame,text='Edit', command=self.modal_open(commands[a]))
                modal.pack(side=LEFT)
    
    def load_parameter_file(self, filepath):
        try:
            f = open(filepath)
            params = yaml.load(f)

            commands = self.commands
            errorOccured = False
            send_params = []

            try:
                for a in commands:
                    command = commands[a]
                    if command['id_name'] in params:
                        name = command['id_name']
                        value = params[name]
                        # TODO: check validate
                        send_params.append({'method':command['method'], 'value': value})
                        print(name + ' : ' + str(value))
                        
                        logger.info("Load parameter file: %s" %(name + ' : ' + str(value)))

            except Exception as e:
                errorOccured = True
                messagebox.showerror('exception',str(e))
                logger.error("exception: %s" %str(e) )

            if not errorOccured:             
                try:
                    for send_param in send_params:
                        method = "self.amaretto.set_" + send_param['method']+"(" + str(amarettoGuiInfo.targetDevId) + ", " + str(send_param['value']) +")"
                        eval(method)
                    
                    self.show_params()
                except MCPError as e:
                    messagebox.showerror('communication exception', str(e))
                    logger.error("communication exception: %s" %str(e) )
                except Exception as e:
                    messagebox.showerror('exception',str(e))
                    logger.error("exception: %s" %str(e) )
        except Exception as e:
            messagebox.showerror('exception',str(e))
            logger.error("exception: %s" %str(e) )

    def load_servo_params(self):
        params_filetypes = [('yaml files', '.yaml .yml')]
        answer = filedialog.askopenfilename(parent=self,
                                            initialdir=os.getcwd(),
                                            title="Please select a parameter file:",
                                            filetypes=params_filetypes)

        if not answer:
            return

        logger.info("LoadFile button clicked")
        self.load_parameter_file(answer)
        logger.info("Load parameter: %s" %self.load_parameter_file(answer))
        
    def load_defaut_params(self):
        
        logger.info("LoadDefault button clicked")
        self.load_parameter_file(os.path.join(os.path.dirname(__file__), 'config', 'default_params.yml'))
        
    def save_servo_params(self):
        
        params_filetypes = [('yaml files', '.yaml .yml')]
        file = filedialog.asksaveasfile(mode="w",
                                     parent= self, 
                                     initialdir=os.getcwd(),
                                     title="Select file", 
                                     filetypes= params_filetypes, 
                                     defaultextension=".yml")
        if file is None:
            return

        devId= amarettoGuiInfo.targetDevId
        params= self.amaretto.get_servo_params(devId)
        yaml.dump(params, file, default_flow_style=False)

        file.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse,sys,time
from amarettopy import *


method_subcmds = [
            "hold"
          , "ready"
          , "free"
          , "stop"
          , "clear_fault"
          , "reset_rotation"
          , "set_ref_current"
          , "get_ref_current"
          , "set_current_KP"
          , "get_current_KP"
          , "set_current_KI"
          , "get_current_KI"
          , "set_current_max_Iterm"
          , "get_current_max_Iterm"
          , "set_current_min_Iterm"
          , "get_current_min_Iterm"
          , "set_current_max_limit"
          , "get_current_max_limit"
          , "set_current_min_limit"
          , "get_current_min_limit"
          , "set_ref_velocity"
          , "get_ref_velocity"
          , "set_velocity_KP"
          , "get_velocity_KP"
          , "set_velocity_KI"
          , "get_velocity_KI"
          , "set_velocity_KD"
          , "get_velocity_KD"
          , "set_velocity_max_Iterm"
          , "get_velocity_max_Iterm"
          , "set_velocity_min_Iterm"
          , "get_velocity_min_Iterm"
          , "set_velocity_max_limit"
          , "get_velocity_max_limit"
          , "set_velocity_min_limit"
          , "get_velocity_min_limit"
          , "set_ref_position"
          , "get_ref_position"
          , "set_position_KP"
          , "get_position_KP"
          , "set_position_KI"
          , "get_position_KI"
          , "set_position_KD"
          , "get_position_KD"
          , "set_position_max_Iterm"
          , "get_position_max_Iterm"
          , "set_position_min_Iterm"
          , "get_position_min_Iterm"
          , "set_position_max_limit"
          , "get_position_max_limit"
          , "set_position_min_limit"
          , "get_position_min_limit"
          , "set_position_offset"
          , "get_position_offset"
          , "set_device_id"
          , "get_device_id"
          , "get_firmware_version"
          , "set_position_sys_offset"
          , "get_position_sys_offset"
          , "get_calibration_data"
          , "set_calibration_data"
          , "get_prot_stop_pin_timeout"
          , "set_prot_stop_pin_timeout"
          , "get_stop_error_threshold"
          , "set_stop_error_threshold"
          , "reset"
          , "fault"
          , "debug"
          , "print_log"
          , "clear_log"
          , "get_power_on_time"
          , "clear_power_on_time"
          , "find_device_id"
          , "save_servo_params"
          , "load_servo_params"
          ]

ext_subcmds = [
            "query_servo_status"
          , "dump_query_servo_status"
          , "move"
          , "calib"
          ]

def create_amaretto(args):
    return UnsafeAmarettoPy(baud=args.baud, port=args.port, timeout_ms=args.timeout)


def cmd_query_servo_status(args):
    amaretto = create_amaretto(args)
    (pos, vel, cur, ref, temp, faults) = amaretto.query_servo_status(args.deviceId)
    print("state:", state2str(amaretto.state()))
    print("pos:", pos)
    print("vel:", vel)
    print("pos[deg]:", toDegree(pos))
    print("vel[rpm]:", toRPM(vel))
    print("cur:", cur)
    print("ref:", ref)
    print("temp:", temp)
    print("faults:", faults2str(faults))
    amaretto.close()

def cmd_dump_query_servo_status(args):
    amaretto = create_amaretto(args)
    start = time.time()
    cnt = 0
    duration_sec = args.duration / 1000.0
    sleep_sec = args.sleep / 1000.0
    print("time[ms]", "pos[deg]", "vel[rpm]", "cur")
    while True:
        try:
            t = time.time()
            duration = t - start
            if duration_sec >= 0 and duration > duration_sec and duration_sec >= 0:
                break
            (pos, vel, cur, ref, temp, faults) = amaretto.query_servo_status(args.deviceId)
            print((int((t-start) * 1000), toDegree(pos), toRPM(vel), cur))
            if sleep_sec > 0 :
                time.sleep(sleep_sec)
            cnt += 1
        except TimeoutError:
            print("timeout")
            continue
    print (cnt / duration, "Hz")
    amaretto.close()

def cmd_move(args):
    amaretto = create_amaretto(args)

    start = amaretto.query_servo_status(args.deviceId)[QSSR_IDX_POS]

    if amaretto.state() not in [STATE_READY, STATE_CURRENT_SERVO, STATE_VELOCITY_SERVO, STATE_POSITION_SERVO]:
        print("keep amaretto ready", amaretto.state())
        sys.exit(-1)
    n = int(round(args.period * args.sampling))
    span = args.period / float(n)
    t0 = time.time()
    path = AmarettoPath([(0, start), (args.period, args.goal)], args.acc, args.dec)
    i = 1
    ref = 0
    while True:
        t = time.time()
        if (t - t0) < i*span:
            continue
        tmp = path.interp(t - t0)
        if (tmp is not None):
            ref = int(tmp)
            i = i + 1
        sen = amaretto.set_ref_position(args.deviceId,ref)
        print(t - t0, ref, sen)
        sys.stdout.flush()
        #assert(abs(ref - sen) < 1)


def cmd_calib(args):
    amaretto = create_amaretto(args)
    calibrate(amaretto, args.deviceId, args.cur, args.span)

def common_parser(parser):
    parser.add_argument('-d', '--deviceId', type=int, default=1, help='deviceID(default: 1)')
    parser.add_argument('-p', '--port', type=str, default='/dev/ttyUSB0', help='device file (ex. /dev/ttyUSB0) or COM port number (ex. COM4). default value is /dev/ttyUSB0')
    parser.add_argument('-b', '--baud', type=int, default='115200', help='baudrate (default: 115200)')
    parser.add_argument('-t', '--timeout', type=float, default='3000', help='timeout[ms] (default: 3000)')
    return parser

def main():

    parser = argparse.ArgumentParser(
            prog='amrtctl',
            description='utility program for Amaretto/Buildit protocol communication'
            )

    subparsers = parser.add_subparsers()

    for subcmd in method_subcmds + ext_subcmds:
        subcmd_cli = subcmd.replace("_", '-')
        subparser = common_parser(subparsers.add_parser(subcmd_cli, help='see `{} -h`'.format(subcmd_cli)))
        subparser.set_defaults(subcmd=subcmd)

        if subcmd == 'stop':
            subparser.add_argument('-T', '--stoptimeout', type=int, default=500, help='stop-timeout [ms]')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.stoptimeout])
        elif subcmd == 'set_ref_current':
            subparser.add_argument('ref', type=int, help='current reference value')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.ref])
        elif subcmd == 'set_ref_velocity':
            subparser.add_argument('ref', type=int, help='velocity reference value[1/100rpm]')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.ref])
        elif subcmd == 'set_ref_position':
            subparser.add_argument('ref', type=int, help='position reference value')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.ref])
        elif subcmd.startswith('set_'):
            subparser.add_argument('value', type=float, help='value')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.value])
        elif subcmd == 'fault':
            subparser.add_argument('faults', type=int, help='fault values')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.faults])
        elif subcmd == 'debug':
            subparser.add_argument('mode', type=int, help='mode(0:none, 1:position, 2:velocity)')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.mode])
        elif subcmd == 'find_device_id':
            subparser.set_defaults(args2list=lambda args:[])
        elif subcmd == 'load_servo_params':
            subparser.add_argument('filepath', type=str, help='parameter file path')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.filepath])
        elif subcmd == 'save_servo_params':
            subparser.add_argument('filepath', type=str, help='parameter file path')
            subparser.set_defaults(args2list=lambda args:[args.deviceId, args.filepath])
        elif subcmd == 'dump_query_servo_status':
            subparser.add_argument('-D', '--duration', type=float, default=-1, help='duration [sec]')
            subparser.add_argument('-s', '--sleep', type=float, default=0, help='sleep [sec]')
        elif subcmd == 'move':
            subparser.add_argument('goal', type=float, help='goal position')
            subparser.add_argument('period', type=float, help='period [sec]')
            subparser.add_argument('--acc', type=float, default=0.2, help='acceleration period rate')
            subparser.add_argument('--dec', type=float, default=0.2, help='deceleration period rate')
            subparser.add_argument('-s', '--sampling', type=float, default=100, help='sampling rate[Hz]')
        elif subcmd == 'calib':
            subparser.add_argument('--cur', type=int, default=1500, help='current reference value')
            subparser.add_argument('--span', type=int, default=1000, help='acceleration span [ms]')
        elif subcmd in method_subcmds:
            subparser.set_defaults(args2list=lambda args:[args.deviceId])

    args = parser.parse_args()

    if "subcmd" not in args.__dict__:
        parser.print_help()
        sys.exit(-1)

    if args.subcmd in method_subcmds:
        amaretto = create_amaretto(args)
        invoker = getattr(amaretto, args.subcmd)
        methodarglist = args.args2list(args)
        result = invoker(*methodarglist)
        if result != () and result != None:
            print(result)
        amaretto.close()
    elif args.subcmd == "query_servo_status":
       cmd_query_servo_status(args)
    elif args.subcmd == "dump_query_servo_status":
       cmd_dump_query_servo_status(args)
    elif args.subcmd == "move":
       cmd_move(args)
    elif args.subcmd == "calib":
       cmd_calib(args)


if __name__ == '__main__':
    main()



import os
from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
from amarettopy.gui.graph_tab import GraphTab
from amarettopy.gui.config_tab import ConfigTab
from amarettopy.gui.log_tab import LogTab
from amarettopy.gui.init_tab import InitTab

class MainPanel(Tk.Frame):

    def __init__(self, amaretto, master):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        self.add_tab()

    def add_tab(self, event=None):
        
        note = ttk.Notebook(self, width=600, height=800)
        if int(os.getenv('UNSAFE_AMARETTO', 0)) == 1:
            note.add(InitTab(self.amaretto, note), text = "Init")
        note.add(ConfigTab(self.amaretto, 'servo.yml', True, note), text = "ServoParam")
        note.add(ConfigTab(self.amaretto, 'system.yml', int(os.getenv('UNSAFE_AMARETTO', 0)) == 1, note), text = "SystemParam")
        note.add(LogTab(self.amaretto, note), text = "Log")

        note.add(GraphTab(self.amaretto, note), text = "Graph")

        note.pack(fill="both", expand=True)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
import struct

MSG_MARK = 0xabccba

MSG_HEADER_SIZE = 8

ACK_BIT = 0b10000000

MSG_TYPE_IDX = 5

DEVICE_ID_DEFAULT = 1
DEVICE_ID_MAX     = 127

MCP_SUCCESS                        = 0x00
MCP_INVALID_FORMAT                 = 0x01
MCP_INVALID_CRC                    = 0x02
MCP_INVALID_COMMAND_PAYLOAD_SIZE   = 0x03
MCP_INVALID_MSG_TYPE               = 0x04
MCP_INVALID_COMMAND_PAYLOAD        = 0x05
MCP_INVALID_OPERATION              = 0x06
MCP_INVALID_MSG_SIZE               = 0x07
MCP_RECV_TIMEOUT                   = 0x08
MCP_OUT_OF_POSITION_LIMIT          = 0x09
MCP_ABS_ENCODER_ERROR              = 0x80
MCP_INC_ENCODER_ERROR              = 0x90
MCP_MOTOR_ERROR                    = 0xa0
MCP_OTHER_ERROR                    = 0xff


MSG_TYPE_QUERY_SERVO_STATUS_CMD = 0x01
MSG_TYPE_GET_LOG_INFO_CMD       = 0x05
MSG_TYPE_GET_LOG_CMD            = 0x06
MSG_TYPE_CLEAR_LOG_CMD          = 0x07
MSG_TYPE_GET_LIFE_LOG_CMD       = 0x08
MSG_TYPE_CLEAR_LIFE_LOG_CMD     = 0x09
MSG_TYPE_READY_CMD              = 0x10
MSG_TYPE_FREE_CMD               = 0x11
MSG_TYPE_HOLD_CMD               = 0x12
MSG_TYPE_CLEAR_FAULT_CMD        = 0x13
MSG_TYPE_PROTECTION_STOP_CMD        = 0x14
MSG_TYPE_SET_REF_CURRENT_CMD    = 0x20
MSG_TYPE_GET_REF_CURRENT_CMD    = 0x21
MSG_TYPE_SET_REF_VELOCITY_CMD   = 0x22
MSG_TYPE_GET_REF_VELOCITY_CMD   = 0x23
MSG_TYPE_SET_REF_POSITION_CMD   = 0x24
MSG_TYPE_GET_REF_POSITION_CMD   = 0x25
MSG_TYPE_SET_PARAM_CMD          = 0x30
MSG_TYPE_GET_PARAM_CMD          = 0x31
MSG_TYPE_RESET_ROTATION_CMD     = 0x32
MSG_TYPE_RESET_CMD              = 0x3c
MSG_TYPE_FAULT_CMD              = 0x3d
MSG_TYPE_DEBUG_CMD              = 0x3f
MSG_TYPE_WRONG_CMD              = 0x7f
MSG_TYPE_NACK                   = 0xff

PARAM_ID_CURRENT_KP           = 0x10
PARAM_ID_CURRENT_KI           = 0x11
PARAM_ID_CURRENT_MAX_ITERM    = 0x12
PARAM_ID_CURRENT_MIN_ITERM    = 0x13
PARAM_ID_CURRENT_MAX_LIMIT    = 0x14
PARAM_ID_CURRENT_MIN_LIMIT    = 0x15

PARAM_ID_VELOCITY_KP          = 0x20
PARAM_ID_VELOCITY_KI          = 0x21
PARAM_ID_VELOCITY_KD          = 0x22
PARAM_ID_VELOCITY_MAX_ITERM   = 0x23
PARAM_ID_VELOCITY_MIN_ITERM   = 0x24
PARAM_ID_VELOCITY_MAX_LIMIT   = 0x25
PARAM_ID_VELOCITY_MIN_LIMIT   = 0x26

PARAM_ID_POSITION_KP          = 0x30
PARAM_ID_POSITION_KI          = 0x31
PARAM_ID_POSITION_KD          = 0x32
PARAM_ID_POSITION_MAX_ITERM   = 0x33
PARAM_ID_POSITION_MIN_ITERM   = 0x34
PARAM_ID_POSITION_MAX_LIMIT   = 0x35
PARAM_ID_POSITION_MIN_LIMIT   = 0x36
PARAM_ID_POSITION_OFFSET      = 0x3a

PARAM_ID_DEVICE_ID            = 0x80
PARAM_ID_FIRMWARE_VERSION     = 0x81

PARAM_ID_CALIBRATION          = 0xa0
PARAM_ID_POSITION_SYS_OFFSET  = 0xa1

PARAM_ID_PROTECTION_STOP_PIN_TIMEOUT     = 0xd0
PARAM_ID_STOP_CONTROL_ERROR_THRESHOLD    = 0xd1

LIFE_LOG_PARAM_ID_POWER_ON_TIME = 0x00  

STATE_HOLD = 0
STATE_FREE = 1
STATE_READY = 2
STATE_CURRENT_SERVO = 3
STATE_VELOCITY_SERVO = 4
STATE_POSITION_SERVO = 5
STATE_PROTECTION_STOPPING = 12
STATE_PROTECTION_STOP     = 13
STATE_FAULT_FREE = 14
STATE_FAULT_HOLD = 15
STATE_UNKNOWN=16


SERVO_FAULT_FOC_DURATION = 0x0001
SERVO_FAULT_OVER_VOLT    = 0x0002
SERVO_FAULT_UNDER_VOLT   = 0x0004
SERVO_FAULT_OVER_TEMP    = 0x0008
SERVO_FAULT_BREAK_IN     = 0x0040
SERVO_FAULT_STOP_CONTROL_ERROR   = 0x0100
SERVO_FAULT_STOP_TIMEOUT = 0x0200
SERVO_FAULT_DUMMY        = 0x0800

LOG_RECORED_BYTE_SIZE             = 16
GET_LOG_CMD_MAX_LOG_RECORED_SIZE  = 10

LOG_LEVEL_FATAL = 0x01
LOG_LEVEL_ERROR = 0x02
LOG_LEVEL_WARN  = 0x03
LOG_LEVEL_INFO  = 0x04
LOG_LEVEL_DEBUG = 0x05
LOG_LEVEL_TRACE = 0x06

LOG_GROUP_SYSTEM                         = 0x01
LOG_GROUP_MAIN                           = 0x02
LOG_GROUP_AMARETTO                       = 0x03
LOG_GROUP_HANDLE_CMD                     = 0x04
LOG_GROUP_NACK                           = 0x05
LOG_GROUP_DOMAIN_BLDC_MOTOR_CONTROL      = 0x10
LOG_GROUP_DOMAIN_HALL_EDGE_ALIGNMENT     = 0x11
LOG_GROUP_DOMAIN_LOG                     = 0x12
LOG_GROUP_DOMAIN_MCP_MESSAGE             = 0x13
LOG_GROUP_DOMAIN_MCP_SERVER              = 0x14
LOG_GROUP_DOMAIN_MOTOR_ABS_ENC_ALIGNMENT = 0x15
LOG_GROUP_DOMAIN_MOTOR_INC_ENC_ALIGNMENT = 0x16
LOG_GROUP_PERIPHERAL_BRAKE               = 0x80
LOG_GROUP_PERIPHERAL_COM485              = 0x81
LOG_GROUP_PERIPHERAL_DEBUG               = 0x82
LOG_GROUP_PERIPHERAL_ENCODER             = 0x83
LOG_GROUP_PERIPHERAL_LED                 = 0x84
LOG_GROUP_PERIPHERAL_MRAM                = 0x85
LOG_GROUP_PERIPHERAL_PROTECTION_STOP     = 0x86
LOG_GROUP_PERIPHERAL_SENSOR              = 0x87
LOG_GROUP_PERIPHERAL_STEADY_CLOCK        = 0x88

LOG_SUB_GROUP_SYSTEM_POWER_ON            = 0x01
LOG_SUB_GROUP_SYSTEM_START_UP            = 0x02
LOG_SUB_GROUP_SYSTEM_ERROR               = 0x10
LOG_SUB_GROUP_SYSTEM_ASSERT              = 0x11
LOG_SUB_GROUP_BLDC_FAULT                 = 0x01
LOG_SUB_GROUP_BLDC_PROTECTION_STOP       = 0x02
LOG_SUB_GROUP_MCP_SERVER_UNNOTIFIED_ERROR= 0x01

LOG_RECORD_INFO_BYTE_SIZE   = 16
LOG_RECORD_BYTE_SIZE        = 16
LOG_RECORD_PAYLOAD_SIZE     = 8

#query servo status result index
QSSR_IDX_POS = 0
QSSR_IDX_VEL = 1
QSSR_IDX_CUR = 2
QSSR_IDX_REF = 3
QSSR_IDX_TEMP = 4
QSSR_IDX_FAULTS = 5

def state2str(s):
    if s == STATE_HOLD:
        return "STATE_HOLD"
    elif s == STATE_FREE:
        return "STATE_FREE"
    elif s == STATE_READY:
        return "STATE_READY"
    elif s == STATE_CURRENT_SERVO:
        return "STATE_CURRENT_SERVO"
    elif s == STATE_FAULT_FREE:
        return "STATE_FAULT_FREE"
    elif s == STATE_FAULT_HOLD:
        return "STATE_FAULT_HOLD"
    elif s == STATE_VELOCITY_SERVO:
        return "STATE_VELOCITY_SERVO"
    elif s == STATE_POSITION_SERVO:
        return "STATE_POSITION_SERVO"
    elif s == STATE_PROTECTION_STOPPING:
        return "STATE_PROTECTION_STOPPING"
    elif s == STATE_PROTECTION_STOP:
        return "STATE_PROTECTION_STOP"
    else:
        return "STATE_UNKNOWN"

def faults2str(s):
    ret = []
    if s & SERVO_FAULT_FOC_DURATION:
        ret.append("SERVO_FAULT_FOC_DURATION")
    if s & SERVO_FAULT_OVER_VOLT   :
        ret.append("SERVO_FAULT_OVER_VOLT")
    if s & SERVO_FAULT_UNDER_VOLT  :
        ret.append("SERVO_FAULT_UNDER_VOLT")
    if s & SERVO_FAULT_OVER_TEMP   :
        ret.append("SERVO_FAULT_OVER_TEMP")
    if s & SERVO_FAULT_STOP_TIMEOUT  :
        ret.append("SERVO_FAULT_STOP_TIMEOUT")
    if s & SERVO_FAULT_BREAK_IN    :
        ret.append("SERVO_FAULT_BREAK_IN")
    if s & SERVO_FAULT_STOP_CONTROL_ERROR    :
        ret.append("SERVO_FAULT_STOP_CONTROL_ERROR")
    if s & SERVO_FAULT_STOP_TIMEOUT    :
        ret.append("SERVO_FAULT_STOP_TIMEOUT")

    if len(ret) == 0:
        return "NO_FAULTS"
    else:
        return ", ".join(ret)

def err2str(e):
    if e == MCP_SUCCESS:
        return "success"
    elif e == MCP_INVALID_FORMAT                :
        return "invalid_format"
    elif e == MCP_INVALID_CRC                   :
        return "invalid_crc"
    elif e == MCP_INVALID_COMMAND_PAYLOAD_SIZE  :
        return "invalid_command_payload_size"
    elif e == MCP_INVALID_MSG_TYPE              :
        return "invalid_msg_type"
    elif e == MCP_INVALID_COMMAND_PAYLOAD       :
        return "invalid_command_payload"
    elif e == MCP_INVALID_OPERATION             :
        return "invalid_operation"
    elif e == MCP_INVALID_MSG_SIZE              :
        return "invalid_msg_size"
    elif e == MCP_RECV_TIMEOUT                  :
        return "timeout"
    elif e == MCP_OUT_OF_POSITION_LIMIT         :
        return "out_of_position_limit"
    elif e == MCP_ABS_ENCODER_ERROR             :
        return "abs_encoder_error"
    elif e == MCP_INC_ENCODER_ERROR             :
        return "inc_encoder_error"
    elif e == MCP_MOTOR_ERROR                   :
        return "motor_error"
    elif e == MCP_OTHER_ERROR                   :
        return "other_error"
    else:
        return "unknown"


def cmd2str(c):
    if   c == MSG_TYPE_QUERY_SERVO_STATUS_CMD :
        return "query_servo_status"
    elif c == MSG_TYPE_READY_CMD              :
        return "ready"
    elif c == MSG_TYPE_FREE_CMD               :
        return "free"
    elif c == MSG_TYPE_HOLD_CMD               :
        return "hold"
    elif c == MSG_TYPE_CLEAR_FAULT_CMD        :
        return "clear_fault"
    elif c == MSG_TYPE_PROTECTION_STOP_CMD    :
        return "protection_stop"
    elif c == MSG_TYPE_SET_REF_CURRENT_CMD    :
        return "set_ref_current"
    elif c == MSG_TYPE_GET_REF_CURRENT_CMD    :
        return "get_ref_current"
    elif c == MSG_TYPE_SET_REF_VELOCITY_CMD   :
        return "set_ref_velocity"
    elif c == MSG_TYPE_GET_REF_VELOCITY_CMD   :
        return "get_ref_velocity"
    elif c == MSG_TYPE_SET_REF_POSITION_CMD   :
        return "set_ref_position"
    elif c == MSG_TYPE_GET_REF_POSITION_CMD   :
        return "get_ref_position"
    elif c == MSG_TYPE_RESET_ROTATION_CMD     :
        return "reset_rotation"
    elif c == MSG_TYPE_SET_PARAM_CMD          :
        return "set_param"
    elif c == MSG_TYPE_GET_PARAM_CMD          :
        return "get_param"
    elif c == MSG_TYPE_RESET_CMD              :
        return "reset"
    elif c == MSG_TYPE_FAULT_CMD              :
        return "fault"
    elif c == MSG_TYPE_DEBUG_CMD              :
        return "debug"
    elif c == MSG_TYPE_GET_LOG_INFO_CMD       :
        return "get_log_info"
    elif c == MSG_TYPE_GET_LOG_CMD            :
        return "get_log"
    elif c == MSG_TYPE_CLEAR_LOG_CMD          :
        return "clear_log"
    elif c == MSG_TYPE_GET_LIFE_LOG_CMD       :
        return "get_life_log"
    elif c == MSG_TYPE_CLEAR_LIFE_LOG_CMD     :
        return "clear_life_log"
    else:
        return "unknown(" + hex(c) + ")"


def log_level2str(lv):
    if lv == LOG_LEVEL_FATAL:
        return "FATAL"
    elif lv == LOG_LEVEL_ERROR :
        return "ERROR"
    elif lv == LOG_LEVEL_WARN  :
        return "WARN"
    elif lv == LOG_LEVEL_INFO  :
        return "INFO"
    elif lv == LOG_LEVEL_DEBUG :
        return "DEBUG"
    elif lv == LOG_LEVEL_TRACE :
        return "TRACE"
    else:
        return "UNKNOWN"

def log_record2str(lv, g,sg,c,payload):

    s0 = hex(g)
    s1 = hex(sg)
    s2 = hex(c)
    s3 = "[" + " ".join(['0x{:02x}'.format(c) for c in payload]) + "]"

    if g == LOG_GROUP_SYSTEM                           :
        s = "system"
        if   sg == LOG_SUB_GROUP_SYSTEM_POWER_ON            :
            return (s, "power_on", s2, s3)
        elif sg == LOG_SUB_GROUP_SYSTEM_START_UP            :
            return (s, "start_up", s2, s3)
        elif sg == LOG_SUB_GROUP_SYSTEM_ERROR               :
            return (s, "error", s2, s3)
        elif sg == LOG_SUB_GROUP_SYSTEM_ASSERT              :
            if lv == LOG_LEVEL_FATAL:
                hd = struct.unpack('<6sH', payload)
                print(hd)
                filename = hd[0].decode('ascii')
                fileline = hd[1]
                ext = "(.c)" if c == 0 else "(.h)"
                return (s, "assert", s2, "file:" + filename + ext + ", line:" + str(fileline))
            else:
                return (s, "assert", s2, s3)
        else:
            return (s, s1, s2, s3)
    elif g == LOG_GROUP_MAIN                           :
        return ("main", s1, s2, s3)
    elif g == LOG_GROUP_AMARETTO                       :
        return ("amaretto", s1, s2, s3)
    elif g == LOG_GROUP_HANDLE_CMD                     :
        return ("cmd_handler", 'cmd_' + cmd2str(sg), s2, s3)
    elif g == LOG_GROUP_NACK                     :
        return ("nack", "-", err2str(c), s3)
    elif g == LOG_GROUP_DOMAIN_BLDC_MOTOR_CONTROL      :
        s = "domain_bldcmc"
        if sg == LOG_SUB_GROUP_BLDC_FAULT                 :
            faults= struct.unpack('<H', payload[0:2])[0]
            return (s, "bldc_fault", "-", faults2str(faults))
        elif sg == LOG_SUB_GROUP_BLDC_PROTECTION_STOP       :
            return (s, "bldc_protection_stop", s2, s3)
        else:
            return (s, s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_HALL_EDGE_ALIGNMENT     :
        return ("domain_hall_edge_align", s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_LOG                     :
        return ("domain_log", s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_MCP_MESSAGE             :
        return ("domain_mcp_msg", s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_MCP_SERVER              :
        if sg == LOG_SUB_GROUP_MCP_SERVER_UNNOTIFIED_ERROR:
            return ("domain_mcp_server", "unnotified_error", err2str(c), s3)
        return ("domain_mcp_server", s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_MOTOR_ABS_ENC_ALIGNMENT :
        return ("domain_abs_enc_align", s1, s2, s3)
    elif g == LOG_GROUP_DOMAIN_MOTOR_INC_ENC_ALIGNMENT :
        return ("domain_inc_enc_align", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_BRAKE               :
        return ("peripheral_brake", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_COM485              :
        return ("peripheral_com485", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_DEBUG               :
        return ("peripheral_debug", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_ENCODER             :
        return ("peripheral_encoder", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_LED                 :
        return ("peripheral_led", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_MRAM                :
        return ("peripheral_mram", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_PROTECTION_STOP     :
        return ("peripheral_protection_stop", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_SENSOR              :
        return ("peripheral_sensor", s1, s2, s3)
    elif g == LOG_GROUP_PERIPHERAL_STEADY_CLOCK        :
        return ("peripheral_clock", s1, s2, s3)
    else:
        return (s0, s1, s2, s3)





from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
import serial.tools.list_ports
from tkinter import messagebox
import os
from amarettopy.gui.util  import *
from amarettopy import *
import logging

logger = logging.getLogger("amarettopy")
NO_PORT = "-------------------------------"

class TopPanel(Tk.Frame):
     
    def __init__(self, amaretto, master):
        Tk.Frame.__init__(self, master)

        self.amaretto = amaretto
        self.port = None
        self.baud = 115200

        port_label = Label(self, text="PORT", anchor=W, font=("Arial", 8) )
        port_label.pack(padx=10, side="left")
        self.port0()

        baud_label = Label(self, text="BAUD RATE", anchor=W, font=("Arial", 8) )
        baud_label.pack(padx=10, side="left")
        self.baudrate() 

        devid_label = Label(self, text="DEVICE ID", anchor=W, font=("Arial", 8) )
        devid_label.pack(padx=10, side="left")
        self.device_id_list()

        self.button= Tk.Button(self, text='Connect', command=self.open)
        self.button.pack(padx=10, side="left")

        self.logo_O = PhotoImage(file=os.path.join(os.path.dirname(__file__), 'imgs', "LED-OFF.gif"))
        self.logo_G = PhotoImage(file=os.path.join(os.path.dirname(__file__), 'imgs', "LED-GREEN.gif"))

        self.isConnected = False

        self.connection_label = None
        self.led = None

        self.onDisconnected()

        self.onTick();

    def onConnected(self):

        #logger = logging.getLogger("AmarettoApp.amaretto_top.onConnected")
        logger.info("Connected ")

        if self.connection_label is not None : self.connection_label.forget()
        if self.led is not None : self.led.forget()

        self.connection_label = Label(self, text="CONNECTED({})".format(amarettoGuiInfo.targetDevId), anchor=W, font=("Arial", 8) )
        self.connection_label.pack(padx=10, side="left")

        self.led = Label(self, image=self.logo_G)
        self.led.pack(padx=10, side="left")

        self.button.configure(text='Disconnect', command=self.close)

    def onDisconnected(self):

        logger.info("Disconnected ")    

        if self.connection_label is not None : self.connection_label.forget()
        if self.led is not None : self.led.forget()

        self.connection_label = Label(self, text="DISCONNECTED", anchor=W, font=("Arial", 8) )
        self.connection_label.pack(padx=10, side="left")

        self.led = Label(self, image=self.logo_O)
        self.led.pack(padx=10, side="left")

        self.button.configure(text='Connect', command=self.open)
          
    def updateboxlist(self):
        list= serial.tools.list_ports.comports()
        self.box['values'] = ([NO_PORT] + list)

    def port0(self):
        
        self.box_value = StringVar()
        self.box = ttk.Combobox(self, textvariable=self.box_value, postcommand =self.updateboxlist)
        self.box.bind("<<ComboboxSelected>>",self.on_select) #assign function to combobox
        print(serial.tools.list_ports.comports())
        ports = serial.tools.list_ports.comports()
        self.box['values'] = ([NO_PORT] + ports)
        self.box.current(1 if len(ports) > 0 else 0)
        self.on_select(None)
        self.box.pack(padx=10, side="left")

    def on_select (self, event):
        
        logger.info("Current Port: %s" %self.port)
        
        print("port is called")
        self.port = self.box.get().split(' ')[0]
        print(self.port)

        logger.info("Port is selected")
        logger.info("Port: %s" %self.port)
         

    def open(self):
        

        if (self.port != NO_PORT):

            global amarettoGuiInfo
            amarettoGuiInfo.targetDevId = int(self.devidbox.get())
            tryAmaretto(lambda: self.amaretto.open(self.port, self.baud))
            self.isConnected = self.query() is not None

            if self.isConnected :
                self.onConnected()
                logger.info("onConnected")
            else:
                self.close()
                logger.info("close")
                

    def close(self):
        self.amaretto.close()
        self.onDisconnected()

        logger.info("Close")
        logger.info("------------------------------------------") 

    def baudrate(self):
        self.box1_value = StringVar()
        self.box1 = ttk.Combobox(self, textvariable=self.box1_value, state='readonly')
        self.box1.bind("<<ComboboxSelected>>", self.baud_select)
        self.box1['values'] =("115200", "57600", "38400", "28800", "19200",
                  "14400", "9600", "4800", "2400","1200","600","300")
        self.box1.current(0)  ##Set default baud rate value
        self.box1.pack(padx=10, side="left")

    def baud_select (self, event):
        print("baud_select is called")
        print(self.box1.get())
        
        #logger = logging.getLogger("AmarettoApp.amaretto_top.baud_select")
        logger.info("Baudrate is selected") 
        logger.info("baudrate: %s" %self.box1.get())        

    def device_id_list(self):
        self.devid_value = StringVar()
        self.devidbox = ttk.Combobox(self, textvariable=self.devid_value, state='readonly', width=5)
        self.devidbox.bind("<<ComboboxSelected>>", self.devid_select)
        self.devidbox['values'] =[str(x) for x in range(128)]
        self.devidbox.current(1)  ##Set default baud rate value
        self.devidbox.pack(padx=10, side="left")

    def devid_select (self, event):
        print("devid_select is called")
        print(self.devidbox.get())
        
        #logger = logging.getLogger("AmarettoApp.amaretto_top.baud_select")
        logger.info("devid is selected") 
        logger.info("devid: %s" %self.devidbox.get())        

    def query(self):

        #logger.info("query")
        if (self.amaretto.is_open()):
            return tryAmaretto(lambda: self.amaretto.query_servo_status(amarettoGuiInfo.targetDevId))
            logger.info(amaretto.query_servo_status)
        else:
            return None

    def onTick(self):
           
        if self.isConnected:
            servoStatus = self.query()
            self.isConnected = servoStatus is not None

            if (not self.isConnected) :
                self.close()

            global tickListener
            for l in tickListener:
                l.onTick(servoStatus)

        self.after(1, self.onTick)




from tkinter import messagebox
import sys,os
from amarettopy import *
import logging

logger = logging.getLogger("amarettopy")

faultDict = dict([
            (0x000, "Stable"),
            (0x001, "FOC Duration"),
            (0x002, "Over Voltage"),
            (0x004, "Under Voltage"),
            (0x008, "Over Temperature"),
            (0x040, "Break In"),
            (0x100, "Stop Control Error"),
            (0x200, "Stop Timeout"),
            (0x800, "Unknown Error"),
            ])

class AmarettoGuiInfo:
    targetDevId = 1

amarettoGuiInfo = AmarettoGuiInfo()
tickListener = []

def tryAmaretto(f):
    try:
        return f()
    except MCPError as e:
        messagebox.showerror('communication exception', e.__class__.__name__)

        logger.error('communication exception: %s'%e.__class__.__name__)
    except Exception as e:
        messagebox.showerror('exception',str(e))
        
        logger.error('exception: %s'% str(e))
    return None



from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
import sys
from amarettopy import *
from amarettopy.gui.util  import *
import logging
from tkinter import messagebox

logger = logging.getLogger("amarettopy")

class LogTab(Tk.Frame):
     
    def __init__(self, amaretto, master):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        self.log_menu()
    
    def log_menu(self):
     
        self.logFrame = Frame(self)
        self.logFrame.pack()

        self.refreshButton= Tk.Button(self.logFrame, text='Refresh', command=self.refresh)
        self.refreshButton.grid(row=0, column=0, columnspan=1, padx=5, pady=5)

        self.clearLogButton= Tk.Button(self.logFrame, text='ClearLog', command=self.clear_log)
        self.clearLogButton.grid(row=0, column=1, columnspan=1, padx=5, pady=5)

        self.init_table()

    def init_table(self):
        tree = ttk.Treeview(self)

        tree["columns"] = (0,1,2,3,4,5,6)
        tree["show"] = "headings"

        tree.column(0,width=30,stretch=False)     # i
        tree.column(1,width=90,stretch=False)     # index
        tree.column(2,width=60)                   # logLevel
        tree.column(3,width=100)                  # group
        tree.column(4,width=100)                  # subGroup
        tree.column(5,width=40,stretch=False)     # code
        tree.column(6,width=300)                  # payload

        tree.heading(1,text="Index")
        tree.heading(2,text="LogLevel")
        tree.heading(3,text="Group")
        tree.heading(4,text="SubGroup")
        tree.heading(5,text="Code")
        tree.heading(6,text="Payload")

        tree.pack(fill="both", expand=True)
        self.tableView = tree

    def refresh(self):

        logger.info("refresh log")
        text_log_data= "power_on_time : %d [sec]\n" % (self.get_life_log() / 100)

        n = self.get_log_info()
        i = 0

        self.tableView.delete(*self.tableView.get_children())

        index = 0
        while i < n :
            recs =self.get_log(start_index=i, read_size=min(10, n -i))
            for rec in recs:
                text_log_data = text_log_data + str(rec) + "\n"
                logIndex = rec["index"]
                level    = rec["level"]
                group    = rec["group"]
                subGroup = rec["subGroup"]
                code     = rec["code"]
                payload  = rec["payload"]
 
                self.tableView.insert("","end",values=(index, logIndex, level, group, subGroup, code, payload))
                index += 1
            i +=10

    def get_log_info(self):
        
        logger.info("get_log_info")
        readableSize = tryAmaretto(lambda: self.amaretto.get_log_info(amarettoGuiInfo.targetDevId))
        return readableSize

    def get_log(self, start_index, read_size):
        
        data = tryAmaretto(lambda: self.amaretto.get_log(amarettoGuiInfo.targetDevId, start_index, read_size))
        return data["logRecords"]

    def get_life_log(self):
        return tryAmaretto(lambda: self.amaretto.get_power_on_time(amarettoGuiInfo.targetDevId))

    def clear_log(self):

        res = messagebox.askokcancel('askokcancel','Clear Log?')
        if not res:
            return

        logger.info("Clear log button clicked")
        tryAmaretto(lambda: self.amaretto.clear_log(amarettoGuiInfo.targetDevId))

        self.refresh()



from tkinter import *
import tkinter as tk

from amarettopy.gui.top_panel import TopPanel
from amarettopy.gui.side_panel import SidePanel
from amarettopy.gui.main_panel import MainPanel

import sys,os
from amarettopy import *

import time
import logging
import logging.handlers as handlers

logger = logging.getLogger("amarettopy")

class AmarettoGUI(tk.Frame):
    def __init__(self, amaretto, master=None):
        tk.Frame.__init__(self, master)
        self.amaretto = amaretto
        self.pack(expand = True, fill = BOTH)
        self.make_root()
        self.text_log_file()


    def make_root(self):

        self.master.title("AmarettoPy")

        if os.name == 'nt':
            root = PanedWindow(self, sashwidth = 2, orient=VERTICAL, width=1000, height=820)
            bottom = PanedWindow(root, sashwidth = 2)
        else:
            root = PanedWindow(self, sashwidth = 2, orient=VERTICAL, width=1000, height=1000)
            bottom = PanedWindow(root, sashwidth = 2, width=1000, height=800)
        
        root.pack(expand = True, fill = BOTH)

        main = Label(bottom, text = 'panedwindow\nmain', bg = 'yellow')        
    
        side = SidePanel(self.amaretto, root)
        main = MainPanel(self.amaretto, root)


        bottom.add(side, width=200)         
        bottom.add(main)
        
        root.add(TopPanel(self.amaretto, root), height=50)
        root.add(bottom)


    def text_log_file(self):
        
        logfile_name = "amaretto-gui.log"

        #Create logger
        logger.setLevel(logging.INFO)

        FORMAT =logging.Formatter("[%(asctime)s] %(levelname)s: %(filename)s:%(lineno)s: %(message)s")
        
        #handler = handlers. TimedRotatingFileHandler(logfile_name, when="M",  interval=5, backupCount=20 )
        handler = handlers.RotatingFileHandler(logfile_name, maxBytes=1000000,  backupCount=0 )
    
        handler.setLevel(logging.DEBUG)
        handler.setFormatter(FORMAT)
  
        errorlog = handlers.RotatingFileHandler('error.log', maxBytes=1000000, backupCount=0)
        errorlog.setLevel(logging.ERROR)
        errorlog.setFormatter(FORMAT)
        
        logger.addHandler(handler)
        logger.addHandler(errorlog)
        
        logger.info("Amaretto-GUI STARTED")
        msg = "LOG DATA is gathering ..."
        logger.info(msg)       
        

def main():
    amaretto = UnsafeAmarettoPy(baud=115200,port=None,timeout_ms=3000)
    global targetDevId
    targetDevId = 1
    app = AmarettoGUI(amaretto)
    app.mainloop()

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# -*- coding: utf-8 -*-

from amarettopy.lib import *
from amarettopy.const import *
import sys,math
from pprint import pprint

# ノイジーなので2分探索やめておく

def _cost(amaretto, devId, cur, x, span_ms = 1000):

    try:
        amaretto.set_calibration_data(devId, x)
        amaretto.ready(devId)

        amaretto.set_ref_current(devId, cur)
        time.sleep(span_ms/1000)
        vplus = amaretto.query_servo_status(devId)[QSSR_IDX_VEL]
        amaretto.set_ref_current(devId, 0)
        time.sleep(0.5)
        amaretto.set_ref_current(devId, -cur)
        time.sleep(span_ms/1000)
        vminus = amaretto.query_servo_status(devId)[QSSR_IDX_VEL]
        amaretto.set_ref_current(devId, 0)
        time.sleep(0.5)

        amaretto.stop(devId) # out-of-control
        amaretto.hold(devId)

        if vplus <= 0 or vminus >= 0:
            print(x, vplus, vminus, "N/A")
            return math.inf
        else:
            ret = abs(vplus + vminus) / abs(vplus - vminus)
            print(x, vplus, vminus, abs(vplus + vminus), abs(vplus - vminus), ret)
            return  ret

    except InvalidOperationError:
        print("invalid operation")
        print(amaretto.query_servo_status(devId))
        return math.inf
    except WaitUntilTimeoutError:
        print("timeout")
        print(amaretto.query_servo_status(devId))
        return math.inf


def calibrate(amaretto, devId, current=2000, span_ms=1000):
    """

    キャリブレーションを行う

    Parameters
    ----------
    devId : int
        デバイスID
    current: int
        指令電流値
    """

    # rough search

    resolution = 256
    rough_skip = 16

    assert(resolution % rough_skip == 0)

    amaretto.ready(devId)

    rough_search_result = []

    for x in range(0, resolution, rough_skip):
        rough_search_result.append((x, _cost(amaretto, devId, current, x, span_ms)))

    rough_best = sorted(rough_search_result, key= lambda a : a[1])[0]

    #pprint(candidates)
    rough_best_idx = rough_best[0]
    rough_best_val = rough_best[1]

    if rough_best_val == math.inf:
        sys.exit("candidates are not found")

    end_idx   = (rough_best_idx + rough_skip) % resolution
    start_idx = (rough_best_idx - rough_skip) % resolution

    start_val = [v for (i,v) in rough_search_result if i == start_idx][0]
    end_val   = [v for (i,v) in rough_search_result if i == end_idx][0]

    if start_val == math.inf:
        start_idx = rough_best_idx
        start_val = rough_best_val
    if end_val == math.inf:
        end_idx = rough_best_idx
        end_val = rough_best_val

    if start_idx == end_idx:
        sys.exit("calibration error: too small search range")

    fine_search_result = [(start_idx, start_val), (end_idx, end_val)]

    idx = start_idx

    while idx != end_idx:
        idx = (idx + 1) % resolution
        fine_search_result.append((idx, _cost(amaretto, devId, current, idx, span_ms)))

    best = sorted(fine_search_result, key= lambda a : a[1])[0]

    print("result: ", best)

    amaretto.set_calibration_data(devId, best[0])





import sys,os
from tkinter import *
import tkinter.ttk as ttk
import tkinter as Tk
from amarettopy.gui.util  import *
from tkinter import simpledialog
from amarettopy import *
from tkinter import messagebox
import logging

logger = logging.getLogger("amarettopy")

class FaultStatusFrame(Tk.Frame):

    def __init__(self, amaretto, master=None):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        self.logo_R = PhotoImage(file=os.path.join(os.path.dirname(__file__), 'imgs', "LED-RED.gif"))
        self.logo_O = PhotoImage(file=os.path.join(os.path.dirname(__file__), 'imgs', "LED-OFF.gif"))
        self.logo_G = PhotoImage(file=os.path.join(os.path.dirname(__file__), 'imgs', "LED-GREEN.gif"))

        self.isConnected = False
        self.faults  = 0


        for i,(k,v) in enumerate(faultDict.items()):

            stat =Label(self, text=v, font=("Arial Bold", 8))
            stat.grid(column=2, row=i+1, sticky=N+S+W)

            led = Label(self, image=self.logo_O)
            led.grid(column=1, row=i+1, sticky=N+S+E)


    def lightOff(self):
        led = Label(self, image=self.logo_O)
        led.grid(column=1, row=1, sticky=N+S+E)
        for i,(k,v) in enumerate(faultDict.items()):
            led = Label(self, image=self.logo_O)
            led.grid(column=1, row=i+1, sticky=N+S+E)

    def lightOn(self):
        if self.faults == 0:
            led = Label(self, image=self.logo_G)
            led.grid(column=1, row=1, sticky=N+S+E)
            for i,(k,v) in enumerate(faultDict.items()):
                if i == 0: continue
                led = Label(self, image=self.logo_O)
                led.grid(column=1, row=i+1, sticky=N+S+E)
        else:
            for i,(k,v) in enumerate(faultDict.items()):
                if (k & self.faults) > 0:
                    led = Label(self, image=self.logo_R)
                    led.grid(column=1, row=i+1, sticky=N+S+E)
                else:
                    led = Label(self, image=self.logo_O)
                    led.grid(column=1, row=i+1, sticky=N+S+E)

    def onTick(self, servoStatus):
        if (self.isConnected):
            if (servoStatus is None):
                self.lightOff()
            elif (self.faults != servoStatus[5]):
                self.faults = servoStatus[5]
                self.lightOn()
        elif (servoStatus is not None):
            self.lightOn()

        self.isConnected = servoStatus is not None

class ModeControlFrame(Tk.Frame):
    
    def __init__(self, amaretto, master=None):
        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto
        self.currentState = STATE_UNKNOWN
        self.currentTemp = "?"

        self.clearButton= Tk.Button(self, text='ClearFault', width= "10", command=self.clear)
        self.clearButton.pack(pady=5)

        self.holdButton= Tk.Button(self, text='Hold', width= "10", command=self.hold)
        self.holdButton.pack(pady=5)

        self.freeButton= Tk.Button(self, text='Free', width= "10", command=self.free)
        self.freeButton.pack(pady=5)

        self.readyButton= Tk.Button(self, text='Ready', width= "10", command=self.ready)
        self.readyButton.pack(pady=5)

        self.modeSelector()

        self.refvalText = Tk.Entry(self, width=10)
        self.refvalText.insert(Tk.END,"0")
        self.refvalText.pack(pady=5)

        self.speedLimit = IntVar()
        self.speedLimit.set(60)


        self.speedSlider = Tk.Scale(self, variable=self.speedLimit, orient=HORIZONTAL, from_=1, to=60, command=self.set_speed_limit)
        self.speedSlider.pack(pady=5)

        self.button= Tk.Button(self,text='SetRef', width= "5", command=self.set_ref)
        self.button.pack(pady=5)

        self.stopButton= Tk.Button(self, text='Stop', width= "10", command=self.stop)
        self.stopButton.pack(pady=5)

        self.resetRot= Tk.Button(self, text='ResetRot', width= "10", command=self.reset_rotation)
        self.resetRot.pack(pady=5)

        self.statText = StringVar()
        self.statText.set(state2str(self.currentState))
        self.stat = Label(self, textvariable=self.statText, font=("Arial Bold", 9))
        self.stat.pack(pady=5)

        self.tempText = StringVar()
        self.tempText.set((str(self.currentTemp) + "°C"))

        self.temp =Label(self, textvariable=self.tempText, font=("Arial Bold", 9), fg="black")
        self.temp.pack(pady=5)

    def set_speed_limit(self, v):
        if self.amaretto.is_open():
            logger.info("sliderclicked ")
            tryAmaretto(lambda: self.amaretto.set_velocity_max_limit(amarettoGuiInfo.targetDevId, fromRPM(self.speedLimit.get())))
            tryAmaretto(lambda: self.amaretto.set_velocity_min_limit(amarettoGuiInfo.targetDevId, fromRPM(-self.speedLimit.get())))


    def hold(self):
        
        logger.info("Hold button clicked ")
        tryAmaretto(lambda: self.amaretto.hold(amarettoGuiInfo.targetDevId))

    def free(self):
        
        logger.info("Free button clicked ")
        tryAmaretto(lambda: self.amaretto.free(amarettoGuiInfo.targetDevId))
        logger.info("State free")
    
    def ready(self):
        
        logger.info("Ready button clicked ")
        tryAmaretto(lambda: self.amaretto.ready(amarettoGuiInfo.targetDevId))

    def clear(self):
        
        logger.info("Clear button clicked ")
        tryAmaretto(lambda: self.amaretto.clear_fault(amarettoGuiInfo.targetDevId))

    def stop(self):
        
        logger.info("Stop button clicked ")
        tryAmaretto(lambda: self.amaretto.stop(amarettoGuiInfo.targetDevId))

    def reset_rotation(self):
        
        logger.info("ResetRot button clicked ")
        tryAmaretto(lambda: self.amaretto.reset_rotation(amarettoGuiInfo.targetDevId))

    def set_ref(self):
        if not self.amaretto.is_open():
            messagebox.showerror('error', 'no connection   ')
            logger.error('Connection Error')
            return

        ret = None
        if self.mode.get() == 0:
            v = int(self.refvalText.get())
            ret = tryAmaretto(lambda: self.amaretto.set_ref_current(amarettoGuiInfo.targetDevId, v))
            logger.info("Current: %s" %v)
        elif self.mode.get() == 1:
            v = float(self.refvalText.get())
            ret = tryAmaretto(lambda: self.amaretto.set_ref_velocity(amarettoGuiInfo.targetDevId, fromRPM(v)))
            logger.info("velocity: %s" %v)
        elif self.mode.get() == 2:
            v = float(self.refvalText.get())
            ret = tryAmaretto(lambda: self.amaretto.set_ref_position(amarettoGuiInfo.targetDevId, fromDegree(v)))
            logger.info("Position: %s" %v)

    def modeSelector(self):

        self.mode = IntVar()
        self.mode.set(2)

        self.radio_cur = ttk.Radiobutton(self, value = 0, variable = self.mode, text = "current", width= "15")
        self.radio_cur.pack(pady=2)
        self.radio_vel = ttk.Radiobutton(self, value = 1, variable = self.mode, text = "velocity[rpm]", width= "15")
        self.radio_vel.pack(pady=2)
        self.radio_pos = ttk.Radiobutton(self, value = 2, variable = self.mode, text = "position[deg]", width= "15")
        self.radio_pos.pack(pady=2)
        logger.info("Mode Selector")

    def updateStatus(self):
        self.statText.set(state2str(self.currentState))
        logger.info("current servo status: %s" %self.currentState)

    def updateTemp(self):
        self.tempText.set(str(self.currentTemp) + "°C")

    def onTick(self, servoStatus):
        if servoStatus is not None:
            newState = self.amaretto.state()
            logger.info('Servo status: %s'%newState)

            if (servoStatus[4] != self.currentTemp):
                self.currentTemp = servoStatus[4]
                self.updateTemp()
                logger.info('CurrentTemp: %s'%self.currentTemp)

            if newState != self.currentState:
                self.currentState = newState
                self.updateStatus()
                logger.info('Newstate-updatestatus %s'%self.updateStatus())

class SidePanel(Tk.Frame):

    def __init__(self, amaretto, master):
        Tk.Frame.__init__(self, master)

        global tickListener
        tickListener.append(self)

        self.modeControl = ModeControlFrame(amaretto, self)
        self.modeControl.pack()
        self.faultsStatus = FaultStatusFrame(amaretto, self)
        self.faultsStatus.pack()

    def onTick(self, servoStatus):
        self.modeControl.onTick(servoStatus)
        self.faultsStatus.onTick(servoStatus)





# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="amarettopy",
    version="0.0.5",
    author="SR",
    author_email="sr-dev@smartrobotics.jp",
    description="python API for Buildit Actuator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['crc8','numpy','pyserial', 'matplotlib', 'numpy', 'pyyaml'],
    url="https://github.com/Smartrobotics/AmarettoPy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_data={'amarettopy': ['gui/imgs/*.gif', 'gui/config/*.yml'] },
    entry_points={
        'console_scripts':[
            'amrtctl = amarettopy.cli.amrtctl:main',
        ],
        'gui_scripts':[
            'amrtctl-gui = amarettopy.gui.amrtctl:main',
        ],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Buildit Actuator を制御する為のPython3用ライブラリ

インストール方法

```
pip install amarettopy
```

"""

import time
import sys, os, struct
import serial
import crc8
from amarettopy.const import *
import yaml

def toDegree(rawPosition):
    """位置情報の単位を[360/65536 * 度]から[度]に変換する
    """
    return round(rawPosition * 360.0 / float(0x10000), 2)

def fromDegree(degree):
    """位置情報の単位を[度]から[360/65536 * 度]に変換する
    """
    return round(degree/360.0 * 0x10000)

def toRPM(rawVelocity):
    """速度情報の単位を[1/100 * rpm]から[rpm]に変換する
    """
    return rawVelocity / float(100)

def fromRPM(rpm):
    """速度情報の単位を[rpm]から[1/100 * rpm]に変換する
    """
    return round(rpm * 100)

class MCPError(Exception):
    """モーター制御プロトコルエラー用例外ベースクラス
    """
    pass

class MCPTransportError(MCPError):
    """通信層でのエラー用例外ベースクラス
    """
    pass

class MCPApplicationError(MCPError):
    """アプリケーション層でのエラー用例外ベースクラス
    """
    pass

class InvalidFormatError(MCPTransportError):
    """不正なメッセージファーマット例外"""
    pass

class InvalidCRCError(MCPTransportError):
    """不正なCRC例外"""
    pass

class UnexpectedMessageTypeError(MCPTransportError):
    """予想外のメッセージタイプ受信例外"""
    pass

class InvalidCommandPayloadSizeError(MCPTransportError):
    """不正なペイロードサイズ例外"""
    pass

class TimeoutError(MCPTransportError):
    """タイムアウト例外"""
    def __init__(self, e):
        self.args = [e]

class InvalidMessageTypeError(MCPApplicationError):
    """不正なメッセージタイプ例外"""
    pass

class InvalidPayload(MCPApplicationError):
    """不正なペイロード例外"""
    pass

class InvalidOperationError(MCPApplicationError):
    """不正な操作例外"""
    def __init__(self, st):
        self.args = [st]

class OutOfPositionLimitError(MCPApplicationError):
    """位置範囲制限外エラー"""
    pass

class OtherError(MCPApplicationError):
    """その他の例外"""

    def __init__(self, e):
        self.args = [e]

class WaitUntilTimeoutError(Exception):
    """wait_untilメソッド用タイムアウト例外"""
    def __init__(self, e):
        self.args = [e]



class AmarettoPy(object):
    """
    モーター制御プロトコルをカプセル化し、Buildit Actuatorと通信する為のクラス

    Examples
    ---------
    >>> from amarettopy import *
    >>> amaretto = AmarettoPy(port="/dev/ttyUSB0", timeout_ms=3000) #for Linux
    >>> amaretto = AmarettoPy(port="COM8", timeout_ms=3000) #for Win
    >>> deviceId = 1
    >>> (position, velocity, current, referenceValue, temperature, faults) = amaretto.query_servo_status(deviceId)
    >>> print(state2str(amaretto.state()))
    >>> #amaretto.clear_fault(deviceId)
    >>> amaretto.ready(deviceId)
    >>> amaretto.set_ref_velocity(deviceId, fromRPM(42.5))
    >>> amaretto.set_ref_position(deviceId, fromDegree(180))

    """

    def _write(self,msg):
        sendedSize = self._ser.write(msg)
        assert sendedSize == len(msg),"serial write error"

    def _read(self,msg_size):
        return self._ser.read(msg_size)

    def _calcCRC(self,s):
        hash = crc8.crc8()
        hash.update(s)

        return int(hash.hexdigest(), 16)


    def __init__(self, port=os.environ.get('AMARETTO_PORT',"/dev/ttyUSB0"), baud=os.environ.get('AMARETTO_BAUD',115200), timeout_ms=3000):
        """

        指定されたパラメーターでシリアルポートをオープンする

        Parameters
        ----------
        baud : int, default 115200
            ボーレート
        port : str, default "/dev/ttyUSB0"
            シリアルポートの名前(例: Linuxなら/dev/ttyUSB0, WindowsならCOM8等)
        timeout_ms : int, default 3000
            タイムアウト(ミリ秒) 負の値であればはタイムアウトしない

        """
        if timeout_ms < 0:
            self._ser = serial.Serial(port,baud)
        else:
            self._ser = serial.Serial(port,baud,timeout=timeout_ms/1000.0)
        if self.is_open():
            self._ser.reset_input_buffer()
            self._ser.reset_output_buffer()
        self._status = None

    def is_open(self):
        """
        シリアルポートがオープンされていればTrue、そうでなければFalseを返す
        """
        return self._ser.is_open

    def open(self, port=os.environ.get('AMARETTO_PORT',"/dev/ttyUSB0"), baud=os.environ.get('AMARETTO_BAUD',115200)):
        """
        シリアルポートをオープンする

        Parameters
        ----------

        baud : int, default 115200
            ボーレート
        port : str, default "/dev/ttyUSB0"
            シリアルポートの名前(例: Linuxなら/dev/ttyUSB0, WindowsならCOM8等)
        """
        self._ser.baudrate = baud
        self._ser.port = port
        self._ser.open()
        self._ser.reset_input_buffer()
        self._ser.reset_output_buffer()

    def close(self):
        """
        シリアルポートをクローズする
        """
        self._ser.close()

    def _receive_ack(self):

        header = self._read(MSG_HEADER_SIZE)
        if len(header) != MSG_HEADER_SIZE:
            raise TimeoutError((header, len(header)))

        (mark1, mark2, mark3, crc, devId, msgType, payloadSize) = struct.unpack('BBBBBBB', header[0:7])
        mark = (mark1 << 16) + (mark2 << 8) + mark3

        if mark != MSG_MARK:
            raise InvalidFormatError

        payload = self._read(payloadSize)

        if len(payload) != payloadSize:
            raise TimeoutError((payload, len(payload)))

        if crc != self._calcCRC(header[4:8]+payload):
            raise InvalidCRCError

        #print "recv:", struct.unpack('B' * len(header), header)

        return (msgType, payload)

    def _nack2exception(self, payload):

        (status,err) = struct.unpack('HB',payload)
        self._status = status
        if err == MCP_INVALID_COMMAND_PAYLOAD:
            return InvalidPayload
        elif err == MCP_INVALID_COMMAND_PAYLOAD_SIZE:
            return InvalidCommandPayloadSizeError
        elif err == MCP_INVALID_OPERATION:
            return InvalidOperationError(state2str(self.state()))
        elif err == MCP_INVALID_MSG_TYPE:
            return InvalidMessageTypeError
        elif err == MCP_OUT_OF_POSITION_LIMIT:
            return OutOfPositionLimitError
        else:
            return OtherError(err)

    def _rpc_raw(self, msg, ackfmt):

        self._write(msg)
        (ackType, payload) = self._receive_ack()

        cmdType = struct.unpack('B'*len(msg), msg)[MSG_TYPE_IDX]

        if ackType == MSG_TYPE_NACK:
            raise self._nack2exception(payload)
        elif ackType == (ACK_BIT | cmdType):
            #print (ackfmt, len(payload), payload)
            bs = struct.unpack('<H'+ackfmt, payload)
            self._status = bs[0]
            return bs[1:]
        else:
            raise UnexpectedMessageTypeError

    def _rpc_log_raw(self, msg, ackfmt):

        self._write(msg)
        (ackType, payload) = self._receive_ack()

        cmdType = struct.unpack('B'*len(msg), msg)[MSG_TYPE_IDX]

        if ackType == MSG_TYPE_NACK:
            raise self._nack2exception(payload)
        elif ackType == (ACK_BIT | cmdType):
            status = struct.unpack('<H', payload[0:2])[0]
            startIndex = struct.unpack('<H', payload[2:4])[0]
            readSize = struct.unpack('<H', payload[4:6])[0]

            logRecords = []
            for i in range(readSize):
                index = 6 + i * LOG_RECORD_BYTE_SIZE
                recordBytes = payload[index:(index+LOG_RECORD_BYTE_SIZE)]
                hd = struct.unpack('<IBBBB', recordBytes[0:LOG_RECORD_BYTE_SIZE-LOG_RECORD_PAYLOAD_SIZE])
                logIndex = hd[0]
                logLevel = hd[1]
                group    = hd[2]
                subGroup = hd[3]
                code     = hd[4]
                payload_data = recordBytes[LOG_RECORD_BYTE_SIZE-LOG_RECORD_PAYLOAD_SIZE:LOG_RECORD_BYTE_SIZE]
                (g,sg,c, data) = log_record2str(logLevel, group, subGroup, code, payload_data)
                logRecord = {'index':logIndex, 'level':log_level2str(logLevel), 'group':g, 'subGroup':sg, 'code':c, 'payload':data}
                # print(logRecord)
                logRecords.append(logRecord)

            self._status = status

            logData = {'startIndex':startIndex, 'readSize':readSize, 'logRecords':logRecords}
            #print('logInfo : ', str(logData))
            return logData
        else:
            raise UnexpectedMessageTypeError

    def _rpc_make_msg(self, msgType, devId, payloadSize = None, mark=MSG_MARK, crc = None, payload=bytes()):

        if payloadSize == None:
            payloadSize = len(payload)

        m = struct.pack('BBB', mark >> 16 & 0xff,  mark >> 8 & 0xff, mark & 0xff)

        #print(devId, msgType, payloadSize, 0)
        header = struct.pack('BBH', devId, msgType, payloadSize)

        if crc == None:
            crc = struct.pack('B', self._calcCRC(header + payload))
        else:
            crc = struct.pack('B', crc)

        msg = m + crc + header + payload

        return msg

    def _rpc(self, msgType, devId, payloadSize=None, mark=MSG_MARK, crc=None, payload=bytes(), ackfmt=''):

        msg = self._rpc_make_msg(msgType, devId, payloadSize, mark, crc, payload)

        return self._rpc_raw(msg, ackfmt)

    def _rpc_log(self, msgType, devId, payloadSize=None, mark=MSG_MARK, crc=None, payload=bytes(), ackfmt=''):

        msg = self._rpc_make_msg(msgType, devId, payloadSize, mark, crc, payload)

        return self._rpc_log_raw(msg, ackfmt)

    def servo_cmds_are_acceptable(self):
        return self.state() in [STATE_READY, STATE_CURRENT_SERVO, STATE_VELOCITY_SERVO, STATE_POSITION_SERVO]

    def has_reference(self):
        return self.state() in [STATE_CURRENT_SERVO, STATE_VELOCITY_SERVO, STATE_POSITION_SERVO]

    def unnotified_error(self):
        """
        最新の応答に含まれる未通知のエラーが発生していれば1、そうでなければ0(ただし、一度も通信していなければNone)
        """
        if self._status is None:
            return None
        return (self._status & 0b10000) > 0

    def state(self):
        """
        最新の応答に含まれるBuildit Actuatorの状態を返す(ただし、一度も通信していなければNone)
        """
        if self._status is None:
            return None
        return (self._status) & 0xf

    def wait_until(self, devId, pred, timeout_ms = -1):
        """
        Buildit Actuatorに対するquery_servo_statusの結果が述語を成立させるまで待つ

        Parameters
        ----------

        devId : int
            デバイスID
        pred : function
            第一引数がBuildit Actuatorの状態、第二引数がquery_servo_statusの結果に対する述語
        timeout_ms : int
            タイムアウト値(ミリ秒)

        """

        starttime = time.time()
        timeout_sec = timeout_ms / 1000.0

        while True:
            q = self.query_servo_status(devId)
            if pred(self.state(), q):
                break
            if timeout_sec >= 0 and time.time() - starttime > timeout_sec:
                raise WaitUntilTimeoutError((state2str(self.state()), q))

    def wait_until_state(self, devId, targetState, timeout_ms = -1):
        """
        状態遷移完了をポーリングで待つ

        Parameters
        ----------

        devId : int
            デバイスID
        s : int
            状態ID
        timeout_ms : int
            タイムアウト値(ミリ秒)

        """

        self.wait_until(devId, lambda s,q: s == targetState, timeout_ms)

    # フィールドアクセス可能なオブジェクトにして返すべき
    def query_servo_status(self,devId):
        """
        Buildit Actuatorの状態を問い合わせる

        Parameters
        ----------

        devId : int
            デバイスID

        Returns
        -------
        pos : int
            センサで計測された位置

        vel : int
            センサで計測された速度

        cur : int
            センサで計測された電流値

        ref : int
            現在の制御指令値

        temp : int
            センサで計測された温度

        faults : int
            フォルトフラグ(SERVO_FAULT_XXXX)

        """

        return self._rpc(MSG_TYPE_QUERY_SERVO_STATUS_CMD, devId, ackfmt='ihhiBH')

    def hold(self, devId):
        """
        Buildit Actuatorをhold状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_HOLD_CMD, devId, ackfmt='')

    def ready(self, devId):
        """
        Buildit Actuatorをready状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_READY_CMD, devId, ackfmt='')

    def forceReady(self, devId):
        """
        Buildit Actuatorをready状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        """
        self.query_servo_status(devId)
        s = self.state()

        if s in [STATE_CURRENT_SERVO, STATE_VELOCITY_SERVO, STATE_POSITION_SERVO]:
            self.stop(devId)
            self.wait_until_state(devId, STATE_READY, timeout_ms = 200)
        elif s == STATE_READY :
            return
        elif s in [STATE_FAULT_FREE, STATE_FAULT_HOLD]:
            self.clear_fault(devId)
            self.ready(devId)
        else:
            self.ready(devId)

    def free(self, devId):
        """
        Buildit Actuatorをfree状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_FREE_CMD, devId, ackfmt='')

    def stop(self, devId, timeout_ms=500):
        """
        保護停止

        Parameters
        ----------

        devId : int
            デバイスID

        timeout_ms : int
            タイムアウト[ミリ秒]

        """
        return self._rpc(MSG_TYPE_PROTECTION_STOP_CMD, devId, payload=struct.pack('<H', timeout_ms), ackfmt='')

    def clear_fault(self, devId):
        """
        フォルトフラグをクリアする

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_CLEAR_FAULT_CMD, devId, ackfmt='')

    def reset_rotation(self, devId):
        """
        Buildit Actuatorの累計回転数を初期化する

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_RESET_ROTATION_CMD, devId, ackfmt='')

    def set_ref_current(self, devId, cur):
        """
        電流指令値を設定した上で電流制御状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        cur : int
            電流指令値

        Returns
        -------
        cur : int
            センサで計測された電流値


        """
        return self._rpc(MSG_TYPE_SET_REF_CURRENT_CMD, devId, payload=struct.pack('h', int(cur)), ackfmt='h')[0]

    def get_ref_current(self, devId):
        """
        電流指令値を取得する
        """
        return self._rpc(MSG_TYPE_GET_REF_CURRENT_CMD, devId, ackfmt='h')[0]

    def set_current_KP(self, devId, v):
        """
        電流制御用比例ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_CURRENT_KP, int(v)), ackfmt='')

    def get_current_KP(self, devId):
        """
        電流制御用比例ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_KP), ackfmt='h')[0]

    def set_current_KI(self, devId, v):
        """
        電流制御用積分ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_CURRENT_KI, int(v)), ackfmt='')

    def get_current_KI(self, devId):
        """
        電流制御用積分ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_KI), ackfmt='h')[0]

    def set_current_max_Iterm(self, devId, v):
        """
        電流制御用積分項の上限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_CURRENT_MAX_ITERM, int(v)), ackfmt='')

    def get_current_max_Iterm(self, devId):
        """
        電流制御用積分項の上限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_MAX_ITERM), ackfmt='i')[0]

    def set_current_min_Iterm(self, devId, v):
        """
        電流制御用積分項の下限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_CURRENT_MIN_ITERM, int(v)), ackfmt='')

    def get_current_min_Iterm(self, devId):
        """
        電流制御用積分項の下限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_MIN_ITERM), ackfmt='i')[0]

    def set_current_max_limit(self, devId, v):
        """
        電流指令値の上限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_CURRENT_MAX_LIMIT, int(v)), ackfmt='')

    def get_current_max_limit(self, devId):
        """
        電流指令値の上限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_MAX_LIMIT), ackfmt='h')[0]

    def set_current_min_limit(self, devId, v):
        """
        電流指令値の下限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_CURRENT_MIN_LIMIT, int(v)), ackfmt='')

    def get_current_min_limit(self, devId):
        """
        電流指令値の下限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CURRENT_MIN_LIMIT), ackfmt='h')[0]

    def set_ref_velocity(self, devId, vel):
        """
        速度指令値を設定した上で速度制御状態に遷移させる

        Parameters
        ----------

        devId : int
            デバイスID

        vel : int
            速度指令値[1/100 * rpm]。符号あり16bit整数値。

        Returns
        -------
        vel : int
            センサで計測された速度[1/100 * rpm]。符号あり16bit整数値。


        """
        return self._rpc(MSG_TYPE_SET_REF_VELOCITY_CMD, devId, payload=struct.pack('h', int(vel)), ackfmt='h')[0]

    def get_ref_velocity(self, devId):
        """
        速度指令値[1/100 * rpm]を取得する 。符号あり16bit整数値。
        """
        return self._rpc(MSG_TYPE_GET_REF_VELOCITY_CMD, devId, ackfmt='h')[0]

    def set_velocity_KP(self, devId, v):
        """
        速度制御用比例ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_VELOCITY_KP, int(v)), ackfmt='')

    def get_velocity_KP(self, devId):
        """
        速度制御用比例ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_KP), ackfmt='h')[0]

    def set_velocity_KI(self, devId, v):
        """
        速度制御用積分ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_VELOCITY_KI, int(v)), ackfmt='')

    def get_velocity_KI(self, devId):
        """
        速度制御用積分ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_KI), ackfmt='h')[0]

    def set_velocity_KD(self, devId, v):
        """
        速度制御用微分ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_VELOCITY_KD, int(v)), ackfmt='')

    def get_velocity_KD(self, devId):
        """
        速度制御用積微分ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_KD), ackfmt='h')[0]

    def set_velocity_max_Iterm(self, devId, v):
        """
        速度制御用積分項の上限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_VELOCITY_MAX_ITERM, int(v)), ackfmt='')

    def get_velocity_max_Iterm(self, devId):
        """
        速度制御用積分項の上限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_MAX_ITERM), ackfmt='i')[0]

    def set_velocity_min_Iterm(self, devId, v):
        """
        速度制御用積分項の下限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_VELOCITY_MIN_ITERM, int(v)), ackfmt='')

    def get_velocity_min_Iterm(self, devId):
        """
        速度制御用積分項の下限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_MIN_ITERM), ackfmt='i')[0]

    def set_velocity_max_limit(self, devId, v):
        """
        速度指令値の上限値を設定する [1/100 * rpm]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_VELOCITY_MAX_LIMIT, int(v)), ackfmt='')

    def get_velocity_max_limit(self, devId):
        """
        速度指令値の上限値を取得する [1/100 * rpm]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_MAX_LIMIT), ackfmt='h')[0]

    def set_velocity_min_limit(self, devId, v):
        """
        速度指令値の下限値を設定する [1/100 * rpm]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_VELOCITY_MIN_LIMIT, int(v)), ackfmt='')

    def get_velocity_min_limit(self, devId):
        """
        速度指令値の下限値を取得する [1/100 * rpm]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_VELOCITY_MIN_LIMIT), ackfmt='h')[0]

    def set_ref_position(self, devId, pos):
        """
        位置指令値を設定した上で位置制御状態に遷移させる
        符号あり32bit整数値。

        Parameters
        ----------

        devId : int
            デバイスID

        pos : int
            位置指令値 [360/65536 * 度]

        Returns
        -------
        pos : int
            センサで計測された位置 [360/65536 * 度]


        """
        return self._rpc(MSG_TYPE_SET_REF_POSITION_CMD, devId, payload=struct.pack('i', int(pos)), ackfmt='i')[0]

    def get_ref_position(self, devId):
        """
        位置指令値を取得する [360/65536 * 度] 符号あり32bit整数値。
        """
        return self._rpc(MSG_TYPE_GET_REF_POSITION_CMD, devId, ackfmt='i')[0]

    def set_position_KP(self, devId, v):
        """
        位置制御用比例ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_POSITION_KP, int(v)), ackfmt='')

    def get_position_KP(self, devId):
        """
        位置制御用比例ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_KP), ackfmt='h')[0]

    def set_position_KI(self, devId, v):
        """
        位置制御用積分ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_POSITION_KI, int(v)), ackfmt='')

    def get_position_KI(self, devId):
        """
        位置制御用積分ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_KI), ackfmt='h')[0]

    def set_position_KD(self, devId, v):
        """
        位置制御用微分ゲインを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_POSITION_KD, int(v)), ackfmt='')

    def get_position_KD(self, devId):
        """
        位置制御用積微分ゲインを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_KD), ackfmt='h')[0]

    def set_position_max_Iterm(self, devId, v):
        """
        位置制御用積分項の上限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_POSITION_MAX_ITERM, int(v)), ackfmt='')

    def get_position_max_Iterm(self, devId):
        """
        位置制御用積分項の上限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_MAX_ITERM), ackfmt='i')[0]

    def set_position_min_Iterm(self, devId, v):
        """
        位置制御用積分項の下限値を設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_POSITION_MIN_ITERM, int(v)), ackfmt='')

    def get_position_min_Iterm(self, devId):
        """
        位置制御用積分項の下限値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_MIN_ITERM), ackfmt='i')[0]

    def set_position_max_limit(self, devId, v):
        """
        位置指令値の上限値を設定する [360/65536 * 度]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_POSITION_MAX_LIMIT, int(v)), ackfmt='')

    def get_position_max_limit(self, devId):
        """
        位置指令値の上限値を取得する [360/65536 * 度]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_MAX_LIMIT), ackfmt='i')[0]

    def set_position_min_limit(self, devId, v):
        """
        位置指令値の下限値を設定する [360/65536 * 度]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bi', PARAM_ID_POSITION_MIN_LIMIT, int(v)), ackfmt='')

    def get_position_min_limit(self, devId):
        """
        位置指令値の下限値を取得する [360/65536 * 度]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_MIN_LIMIT), ackfmt='i')[0]

    def set_position_offset(self, devId, v):
        """
        位置センサのユーザーオフセットを設定する [360/65536 * 度]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<Bh', PARAM_ID_POSITION_OFFSET, int(v)), ackfmt='')

    def get_position_offset(self, devId):
        """
        位置センサのユーザーオフセットを取得する [360/65536 * 度]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_OFFSET), ackfmt='h')[0]

    def set_device_id(self, devId, v):
        """
        デバイスIDを設定する
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<BB', PARAM_ID_DEVICE_ID, int(v)), ackfmt='')

    def get_device_id(self, devId):
        """
        デバイスIDを取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_DEVICE_ID), ackfmt='B')[0]

    def find_device_id(self):
        """
        接続中の応答可能なデバイスのうち最も小さなデバイスIDを返す
        """
        timeout = self._ser.timeout
        self._ser.timeout = 0.05    # sec
        foundId = None
        try:
            for id in range(DEVICE_ID_MAX+1):
                try:
                    self.query_servo_status(devId=id)
                    foundId = id
                    break
                except:
                    pass
        finally:
            self._ser.timeout = timeout

        return foundId

    def get_servo_params(self,devId):
        """
        サーボパラメーターを取得する
        """
    
        params= dict(
        PARAM_ID_CURRENT_KP = self.get_current_KP(devId),
        PARAM_ID_CURRENT_KI = self.get_current_KI(devId),
        PARAM_ID_CURRENT_MAX_ITERM = self.get_current_max_Iterm(devId),
        PARAM_ID_CURRENT_MIN_ITERM = self.get_current_min_Iterm(devId),
        PARAM_ID_CURRENT_MAX_LIMIT = self.get_current_max_limit(devId),
        PARAM_ID_CURRENT_MIN_LIMIT = self.get_current_min_limit(devId),
        
        PARAM_ID_VELOCITY_KP = self.get_velocity_KP(devId),
        PARAM_ID_VELOCITY_KI = self.get_velocity_KI(devId),
        PARAM_ID_VELOCITY_KD = self.get_velocity_KD(devId),
        PARAM_ID_VELOCITY_MAX_ITERM = self.get_velocity_max_Iterm(devId),
        PARAM_ID_VELOCITY_MIN_ITERM = self.get_velocity_min_Iterm(devId),
        PARAM_ID_VELOCITY_MAX_LIMIT = self.get_velocity_max_limit(devId),
        PARAM_ID_VELOCITY_MIN_LIMIT = self.get_velocity_min_limit(devId),
        
        PARAM_ID_POSITION_KP = self.get_position_KP(devId),
        PARAM_ID_POSITION_KI = self.get_position_KI(devId),
        PARAM_ID_POSITION_KD = self.get_position_KD(devId),
        PARAM_ID_POSITION_MAX_ITERM = self.get_position_max_Iterm(devId),
        PARAM_ID_POSITION_MIN_ITERM = self.get_position_min_Iterm(devId),
        PARAM_ID_POSITION_MAX_LIMIT = self.get_position_max_limit(devId),
        PARAM_ID_POSITION_MIN_LIMIT = self.get_position_min_limit(devId),

        PARAM_ID_POSITION_OFFSET = self.get_position_offset(devId)
        )
        #print(params)
        return params

    def set_servo_params(self, devId, params): 
        """
        サーボパラメーターを設定する
        """
       
        self.set_current_KP(devId, params['PARAM_ID_CURRENT_KP'])
        self.set_current_KI(devId, params['PARAM_ID_CURRENT_KI']),
        self.set_current_max_Iterm(devId, params['PARAM_ID_CURRENT_MAX_ITERM']),
        self.set_current_min_Iterm(devId, params['PARAM_ID_CURRENT_MIN_ITERM']),
        self.set_current_max_limit(devId, params['PARAM_ID_CURRENT_MAX_LIMIT']),
        self.set_current_min_limit(devId, params['PARAM_ID_CURRENT_MIN_LIMIT']),
        
        self.set_velocity_KP(devId, params['PARAM_ID_VELOCITY_KP']),
        self.set_velocity_KI(devId, params['PARAM_ID_VELOCITY_KI']),
        self.set_velocity_KD(devId, params['PARAM_ID_VELOCITY_KD']),
        self.set_velocity_max_Iterm(devId, params['PARAM_ID_VELOCITY_MAX_ITERM']),
        self.set_velocity_min_Iterm(devId, params['PARAM_ID_VELOCITY_MIN_ITERM']),
        self.set_velocity_max_limit(devId, params['PARAM_ID_VELOCITY_MAX_LIMIT']),
        self.set_velocity_min_limit(devId, params['PARAM_ID_VELOCITY_MIN_LIMIT']),
        
        self.set_position_KP(devId, params['PARAM_ID_POSITION_KP']),
        self.set_position_KI(devId, params['PARAM_ID_POSITION_KI']),
        self.set_position_KD(devId, params['PARAM_ID_POSITION_KD']),
        self.set_position_max_Iterm(devId, params['PARAM_ID_POSITION_MAX_ITERM']),
        self.set_position_min_Iterm(devId, params['PARAM_ID_POSITION_MIN_ITERM']),
        self.set_position_max_limit(devId, params['PARAM_ID_POSITION_MAX_LIMIT']),
        self.set_position_min_limit(devId, params['PARAM_ID_POSITION_MIN_LIMIT']),

        self.set_position_offset(devId, params['PARAM_ID_POSITION_OFFSET'])

    def save_servo_params(self, devId, fileName=None):
        """
        サーボパラメーターをファイルに保存する
        """
        
        if fileName == None:
            timestr = time.strftime("%Y%m%d-%H%M%S")       
            fileName = "params-"+timestr+".yml"
        
        with open(fileName, "w") as file:
            params = self.get_servo_params(devId)
            yaml.dump(params, file, default_flow_style=False)
        
    def load_servo_params(self, devId, filepath):         
        """
        サーボパラメーターをファイルから読みだす
        """
        
        with open(filepath, "r") as stream:
            try:
                params= yaml.safe_load(stream)
                self.set_servo_params(devId, params)

            except yaml.YAMLError as exc:
                print(exc)    


class UnsafeAmarettoPy(AmarettoPy):
    """
    意図せぬ危険な動作を行う可能性があるメンテナンス用機能
    """

    def __init__(self, port=os.environ.get('AMARETTO_PORT',"/dev/ttyUSB0"), baud=os.environ.get('AMARETTO_BAUD',115200), timeout_ms=3000):
        super().__init__(port, baud, timeout_ms)


    def set_prot_stop_pin_timeout(self, devId, v):
        """
        保護停止ピンによる停止間上限[ms]を設定する
        """
        return self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<BH', PARAM_ID_PROTECTION_STOP_PIN_TIMEOUT, int(v)), ackfmt='')

    def set_stop_control_error_threshold(self, devId, v):
        """
        停止異常閾値を設定する
        """
        return self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<BH', PARAM_ID_STOP_CONTROL_ERROR_THRESHOLD, int(v)), ackfmt='')

    def reset(self, devId):
        """
        ソフトウェアリセットする
        """
        return self._rpc(MSG_TYPE_RESET_CMD, devId, ackfmt='')

    def fault(self, devId, faults):
        """
        任意のフォルトを発生させる

        Parameters
        ----------

        devId : int
            デバイスID

        faults : int
            フォルトIDの論理和


        """
        return self._rpc(MSG_TYPE_FAULT_CMD, devId, payload=struct.pack('H', int(faults)), ackfmt='')

    def debug(self, devId, mode):
        """
        デバッグイベント発生

        Parameters
        ----------

        devId : int
            デバイスID

        mode : int
            モード(0:none, 1:position, 2:velocity)

        """
        return self._rpc(MSG_TYPE_DEBUG_CMD, devId, payload=struct.pack('H', int(mode)), ackfmt='')

    def get_log_info(self, devId):
        """
        ログ情報を取得する

        Parameters
        ----------

        devId : int
            デバイスID

        Returns
        -------
        readableSize : int
            取得可能なログレコード数


        """
        return self._rpc(MSG_TYPE_GET_LOG_INFO_CMD, devId, ackfmt='h')[0]

        """
        ログを出力する

        Parameters
        ----------

        devId : int
            デバイスID

        startIndex : int
            取得するログレコードの開始番号

        readSize : int
            取得するログレコードの個数

        """
    def print_log(self, devId, startIndex = 0, readSize = -1):
        data = self.get_log(devId, startIndex, readSize)
        if len(data['logRecords']) > 0:
            print('startIndex:', data['startIndex'])
            print('readSize:', data['readSize'])
            for record in data['logRecords']:
                #print(record)
                print("{index:10d}:\t{level:s}:\t{group:17s}\t{subGroup:17s}\t{code:s}\t{payload:s}".format(**record))

    def get_log(self, devId, startIndex = 0, readSize = -1):
        """
        ログを取得する

        Parameters
        ----------

        devId : int
            デバイスID

        startIndex : int
            取得するログレコードの開始番号

        readSize : int
            取得するログレコードの個数

        Returns
        -------
        startIndex : int
            取得したログレコードの開始番号

        readSize : int
            取得したログレコードの個数

        logRecords : bytes
            取得したReadSize個分のログレコード
        """
        records = []
        numOfRecords = self.get_log_info(devId)

        if startIndex < 0 :
            startIndex = 0

        if readSize < 0:
            readSize = numOfRecords

        if startIndex + readSize > numOfRecords:
            readSize = min(readSize, numOfRecords - startIndex)

        offset = startIndex
        remain = readSize

        while remain > 0:
            size = min(remain, 10)
            tmp = self._rpc_log(MSG_TYPE_GET_LOG_CMD, devId, payload=struct.pack('hh', int(offset), int(size)), ackfmt='')
            records = records + tmp['logRecords']
            offset = offset + size
            remain = remain - size

        return {'startIndex': startIndex,
                'readSize'  : readSize,
                'logRecords': records}

    def clear_log(self, devId):
        """
        ログを消去する
        """
        return self._rpc(MSG_TYPE_CLEAR_LOG_CMD, devId, ackfmt='')

    def get_power_on_time(self, devId):
        """
        累計通電時間を取得する

        Parameters
        ----------

        devId : int
            デバイスID

        Returns
        -------
        param : int
            累計通電時間

        """
        return self._rpc(MSG_TYPE_GET_LIFE_LOG_CMD, devId, payload=struct.pack('<B', LIFE_LOG_PARAM_ID_POWER_ON_TIME), ackfmt='I')[0]

    def clear_power_on_time(self, devId):
        """
        累計通電時間を0リセットする

        Parameters
        ----------

        devId : int
            デバイスID

        """
        return self._rpc(MSG_TYPE_CLEAR_LIFE_LOG_CMD, devId, payload=struct.pack('<B', LIFE_LOG_PARAM_ID_POWER_ON_TIME), ackfmt='')

    def set_calibration_data(self, devId, v):
        """
        補正値を設定する
        """
        return self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<BH', PARAM_ID_CALIBRATION, int(v)), ackfmt='')

    def get_calibration_data(self, devId):
        """
        補正値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_CALIBRATION), ackfmt='H')[0]

    def get_position_sys_offset(self, devId):
        """
        位置センサのシステムオフセットを取得する [360/65536 * 度]
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_POSITION_SYS_OFFSET), ackfmt='H')[0]

    def get_firmware_version(self, devId):
        """
        ファームウェアのバージョン情報を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_FIRMWARE_VERSION), ackfmt='16s')[0]

    def set_position_sys_offset(self, devId, v):
        """
        位置センサのシステムオフセットを設定する [360/65536 * 度]
        """
        self._rpc(MSG_TYPE_SET_PARAM_CMD, devId, payload=struct.pack('<BH', PARAM_ID_POSITION_SYS_OFFSET, int(v)), ackfmt='')

    def get_prot_stop_pin_timeout(self, devId):
        """
        保護停止ピンによる停止間上限[ms]を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_PROTECTION_STOP_PIN_TIMEOUT), ackfmt='H')[0]

    def get_stop_control_error_threshold(self, devId):
        """
        停止異常閾値を取得する
        """
        return self._rpc(MSG_TYPE_GET_PARAM_CMD, devId, payload=struct.pack('<B', PARAM_ID_STOP_CONTROL_ERROR_THRESHOLD), ackfmt='I')[0]




from tkinter import *
import tkinter as Tk
import tkinter.ttk as ttk
import time
from amarettopy.gui.util  import *
from amarettopy import *

from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
import matplotlib
matplotlib.use("TkAgg")
from matplotlib import pyplot as plt
import matplotlib.animation as animation
import logging

logger = logging.getLogger("amarettopy")

class GraphTab(Tk.Frame):

    def __init__(self, amaretto, master=None):

        Tk.Frame.__init__(self, master)
        self.amaretto = amaretto

        self.t0 = None
        self.fig = None

        global tickListener

        tickListener.append(self)
        
        self.refVal = None
        self.state = STATE_UNKNOWN


    def onTick(self, servoStatus):

        if servoStatus is None:
            self.t0 = None
            self.refVal = None
            
            return

        if self.amaretto.has_reference():
            self.refVal = servoStatus[3]
            s = self.amaretto.state()
            d = dict([(STATE_CURRENT_SERVO, 0),
              (STATE_VELOCITY_SERVO, 1),
              (STATE_POSITION_SERVO, 2),
              ])
            self.servoIdx = d[s]

        else:
            self.refVal = None


        t   = time.time()
        if self.t0 is None:
            self.t0  = t
            self.ts   = []
            self.poss = []
            self.vels = []
            self.curs = []
            if self.fig is not None:
                plt.close(self.fig)
            self.fig, self.ax = plt.subplots(3, 1)

        t0 = self.t0

        cur = servoStatus[2]
        rawvel = servoStatus[1]
        vel = toRPM(rawvel)
        rawpos = servoStatus[0]
        pos = toDegree(rawpos)
        
        ref = servoStatus[3]
        ref = self.ToRef(ref)
        temp = servoStatus[4]
        fault = servoStatus[5] 

        self.ts.append(t-t0)
        self.curs.append(cur)
        self.vels.append(vel)
        self.poss.append(pos)

        XRANGE =  5 # [sec]

        if t - t0 >= XRANGE:
            for x in self.ts[:]:
                if x < t-t0-XRANGE:
                    del(self.ts[0])
                    del(self.curs[0])
                    del(self.vels[0])
                    del(self.poss[0])
                else:
                    break

        lastsens = [cur, vel, pos]
        lastraw = [cur, rawvel, rawpos]
        sens = [self.curs, self.vels, self.poss]
        labels = ['cur', 'vel', 'pos']
        units = [' ', ' rpm', ' deg']
        self.ax[0].tick_params(labelbottom=False)
        self.ax[1].tick_params(labelbottom=False)


        for i in range(3):
            self.ax[i].clear()
            YRANGE2 = 10

            if t - t0 >= XRANGE:
                self.ax[i].set_xlim(t - self.t0 - XRANGE, t-self.t0)
            else:
                self.ax[i].set_xlim(0, XRANGE)

            if self.refVal is not None and self.servoIdx == i:

                yrange = max(YRANGE2, abs(max(sens[i]) - self.ToRef(self.refVal)), abs(min(sens[i]) - self.ToRef(self.refVal)))
                self.ax[i].set_ylim(self.ToRef(self.refVal)-yrange-10, self.ToRef(self.refVal)+yrange+10)
                

                self.ax[i].plot(self.ts, sens[i], 'b', self.ts, [self.ToRef(self.refVal)]*len(self.ts), 'r')
                if labels[i] == 'cur':
                    self.ax[i].set_title( labels[i] + ' {:6}, ref({:6})'.format(lastsens[i], self.ToRef(self.refVal), color='r', fontsize=1) + units[i])
                else:
                    self.ax[i].set_title( labels[i] + ' {0:.2f}[{3}] ({1}), ref({2:.2f})'.format(lastsens[i], lastraw[i], self.ToRef(self.refVal), units[i], color='r', fontsize=1))
            else:
                self.ax[0].set_ylim(-5000,5000)
                self.ax[1].set_ylim(-60,60)
                yrange = max(YRANGE2, abs(max(sens[2]) - pos), abs(min(sens[2]) - pos))
                self.ax[2].set_ylim(pos-yrange, pos+yrange)
                self.ax[i].plot(self.ts, sens[i], 'b')
                if labels[i] == 'cur':
                    self.ax[i].set_title( labels[i] + ' {}'.format(lastsens[i], color='r', fontsize=1) + units[i])
                else:
                    self.ax[i].set_title( labels[i] + ' {0:.2f}[{2}] ({1})'.format(lastsens[i], lastraw[i],  units[i], color='r', fontsize=1))

        self.ax[2].set_xlabel("Time [sec]", fontsize=10)
        
        logger.info("Servo values: cur:%s vel:%s pos:%s ref:%s temp:%s fault:%s " % (cur, vel, pos, ref, temp, fault))
        
        plt.pause(0.01)   

    def ToRef(self, ref):
        state= self.amaretto.state()
        
        if state == STATE_VELOCITY_SERVO:
            return toRPM(ref)
            
        if state == STATE_POSITION_SERVO:  
            return toDegree(ref)

        else: 
            return ref    



